		Title  "Enterprise Nacelle Program"

        list P=16F648A
;
; ------------------
; CONFIGURATION FUSE
; ------------------
;
    INCLUDE "p16f648a.inc"
	__CONFIG _BODEN_OFF & _CP_OFF & _DATA_CP_OFF & _PWRTE_OFF & _WDT_OFF & _LVP_OFF & _MCLRE_OFF & _INTRC_OSC_NOCLKOUT

;;; This program controls LEDs for a model of the Enterprise from
;;; Star Trek.  It operates one nacelle worth of LEDs (10 LEDs) on
;;; lines 0-4 of ports A and B, and also drives the navigational blinkers
;;; on RA7.

Temp equ 0x20
DelayL equ 0x21
DelayM equ 0x22
Count equ 0x23
LedSelect equ 0x24
LedMask equ 0x25
BufferA1 equ 0x26
BufferA2 equ 0x27
BufferB1 equ 0x28
BufferB2 equ 0x29
SpinnerMaskA1 equ 0x2A
SpinnerMaskA2 equ 0x2B
SpinnerMaskB1 equ 0x2C
SpinnerMaskB2 equ 0x2D
Blinky equ 0x2E
RandomSeqPoint equ 0x2F
Blinky1Timer equ 0x30
Blinky2Timer equ 0x31
Blinky3Timer equ 0x32
Blinky4Timer equ 0x33
Blinky5Timer equ 0x34
NavTimer equ 0x35

NavOnPeriod equ .50
NavOffPeriod equ .25


ClearFlags Macro
        bcf STATUS, C
        bcf STATUS, DC
        bcf STATUS, Z
        endm

;
; -------------
; PROGRAM START
; -------------
;
	org 0h			; startup address = 0000

        clrf PORTA
        clrf PORTB
        movlw 0x07
        movwf CMCON
        bcf STATUS, RP1
	bsf STATUS,RP0		; set RP0 for RAM page 1
	movlw b'00000000'		; all PortA = outputs
	movwf TRISA
	movlw b'00000000'		; all PortB = outputs
	movwf TRISB
	bcf STATUS,RP0		; set RP0 for RAM page 0
        movlw 0x0a
        movwf Count
        movlw 0xff
        movwf SpinnerMaskA1
        movwf SpinnerMaskA2
        movwf SpinnerMaskB1
        movwf SpinnerMaskB2
        movlw 0x15
        movwf Blinky
        clrf RandomSeqPoint
        clrf Blinky1Timer
        clrf Blinky2Timer
        clrf Blinky3Timer
        clrf Blinky4Timer
        clrf Blinky5Timer
        clrf NavTimer

MainLoop
        call Delay100
        decf Count, f
        movf Count, w
        movwf LedSelect
        rrf LedSelect, 1

        movlw 0
        movwf SpinnerMaskA1
        movwf SpinnerMaskB1

        movlw 1
        movwf LedMask
        ClearFlags
        movf LedSelect, w
LedMaskLoop
        btfsc STATUS, Z
        goto LedMaskDone
        rlf LedMask, f
        decf LedSelect, f
        goto LedMaskLoop
LedMaskDone
        btfsc Count, 0
        goto SetSpinnerMaskB
SetSpinnerMaskA
        movf LedMask, w
        movwf SpinnerMaskA1
        movwf Temp
        bcf STATUS, C
        rlf LedMask, f
        rlf LedMask, f
        bcf STATUS, C
        rrf Temp, f
        rrf Temp, f
        rrf Temp, f
        movf Temp, w
        iorwf LedMask, w
        andlw 0x1f
        movwf SpinnerMaskB1
        goto SpinnerMaskDone
SetSpinnerMaskB
        movf LedMask, w
        movwf SpinnerMaskB1
        movwf Temp
        bcf STATUS, C
        rlf LedMask, f
        rlf LedMask, f
        rlf LedMask, f
        bcf STATUS, C
        rrf Temp, f
        rrf Temp, f
        movf Temp, w
        iorwf LedMask, w
        andlw 0x1f
        movwf SpinnerMaskA1
SpinnerMaskDone
        ;; SpinnerMaskA2/B2 has the same LEDs blanked out as A1/B1,
        ;; but the neighbors of these blanked-out LEDs are
        ;; blanked-out, too.
        movf SpinnerMaskA1, w
        movwf SpinnerMaskA2
        movwf Temp
        bcf STATUS, C
        rlf Temp, f
        rlf Temp, f
        movf Temp, w
        rlf Temp, f
        iorwf Temp, w
        iorwf SpinnerMaskA2, f

        movf SpinnerMaskA1, w
        movwf Temp
        bcf STATUS, C
        rrf Temp, f
        rrf Temp, f
        movf Temp, w
        rrf Temp, f
        iorwf Temp, w
        iorwf SpinnerMaskA2, f
        movf SpinnerMaskA2, w
        andlw 0x1f
        movwf SpinnerMaskA2

        movf SpinnerMaskB1, w
        movwf SpinnerMaskB2
        movwf Temp
        bcf STATUS, C
        rlf Temp, f
        rlf Temp, f
        movf Temp, w
        rlf Temp, f
        iorwf Temp, w
        iorwf SpinnerMaskB2, f

        movf SpinnerMaskB1, w
        movwf Temp
        bcf STATUS, C
        rrf Temp, f
        rrf Temp, f
        movf Temp, w
        rrf Temp, f
        iorwf Temp, w
        iorwf SpinnerMaskB2, f
        movf SpinnerMaskB2, w
        andlw 0x1f
        movwf SpinnerMaskB2

        movlw 0xff
        xorwf SpinnerMaskA1, f
        xorwf SpinnerMaskA2, f
        xorwf SpinnerMaskB1, f
        xorwf SpinnerMaskB2, f

;;; Now update Blinky
Blinky1Start
        movf Blinky1Timer, f
        btfsc STATUS, Z
        goto Blinky1Reset
        decf Blinky1Timer, f
        goto Blinky1End
Blinky1Reset
        call BlinkyRandom
        movwf Blinky1Timer
        movlw 0x01
        xorwf Blinky, f
        btfss Blinky, 0
        rrf Blinky1Timer, f
Blinky1End

Blinky2Start
        movf Blinky2Timer, f
        btfsc STATUS, Z
        goto Blinky2Reset
        decf Blinky2Timer, f
        goto Blinky2End
Blinky2Reset
        call BlinkyRandom
        movwf Blinky2Timer
        movlw 0x02
        xorwf Blinky, f
        btfss Blinky, 1
        rrf Blinky2Timer, f
Blinky2End

Blinky3Start
        movf Blinky3Timer, f
        btfsc STATUS, Z
        goto Blinky3Reset
        decf Blinky3Timer, f
        goto Blinky3End
Blinky3Reset
        call BlinkyRandom
        movwf Blinky3Timer
        movlw 0x04
        xorwf Blinky, f
        btfss Blinky, 2
        rrf Blinky3Timer, f
Blinky3End

Blinky4Start
        movf Blinky4Timer, f
        btfsc STATUS, Z
        goto Blinky4Reset
        decf Blinky4Timer, f
        goto Blinky4End
Blinky4Reset
        call BlinkyRandom
        movwf Blinky4Timer
        movlw 0x08
        xorwf Blinky, f
        btfss Blinky, 3
        rrf Blinky4Timer, f
Blinky4End

Blinky5Start
        movf Blinky5Timer, f
        btfsc STATUS, Z
        goto Blinky5Reset
        decf Blinky5Timer, f
        goto Blinky5End
Blinky5Reset
        call BlinkyRandom
        movwf Blinky5Timer
        movlw 0x10
        xorwf Blinky, f
        btfss Blinky, 4
        rrf Blinky5Timer, f
Blinky5End

NavStart
        ClearFlags
        movf NavTimer, f
        btfsc STATUS, Z
        goto NavReset
        decf NavTimer, f
        goto NavEnd
NavReset
        movlw 0x80
        xorwf Blinky, f
        movlw NavOnPeriod
        btfss Blinky, 7
        movlw NavOffPeriod
        movwf NavTimer
NavEnd


;;; Now apply SpinnerMask to Blinky to generate BufferA
;;; Bit4 is moved to Bit6 to avoid A4's lame open collectorness.
        movf Blinky, w
        andwf SpinnerMaskA1, w
        movwf BufferA1
        bcf BufferA1, 6
        btfsc BufferA1, 4
        bsf BufferA1, 6

        movf Blinky, w
        andwf SpinnerMaskA2, w
        movwf BufferA2
        bcf BufferA2, 6
        btfsc BufferA2, 4
        bsf BufferA2, 6

;;; BufferB is just SpinnerMaskB
        movf SpinnerMaskB1, w
        movwf BufferB1
        movf SpinnerMaskB2, w
        movwf BufferB2

        ClearFlags
        movf Count, w
        btfss STATUS, Z
        goto MainLoop
        movlw 0x0a
        movwf Count
        goto MainLoop


Delay100
        clrf DelayL
        movlw 0x08
        movwf DelayM
Wait1
        movf DelayL, w
        andlw 0x03
        btfss STATUS, Z
        goto Flicker2
Flicker1
        movf BufferA1, w
        movwf PORTA
        movf BufferB1, w
        movwf PORTB
        goto FlickerDone
Flicker2
        movf BufferA2, w
        movwf PORTA
        movf BufferB2, w
        movwf PORTB
FlickerDone
        decfsz DelayL, 1
        goto Wait1
        decfsz DelayM, 1
        goto Wait1
        return


;;; BlinkyRandom: Sequential calls return successive values from
;;; a stored sequence of pseudo-random values in the range used
;;; for blinky timing (about .1-.4 seconds)
BlinkyRandom
        incf RandomSeqPoint, f
        ClearFlags
        movf RandomSeqPoint, w
        addlw LOW BlinkyTable
        movwf Temp
        movlw HIGH BlinkyTable
        btfsc STATUS, C
        addlw 1

        movwf PCLATH
        movf Temp, w
        movwf PCL

BlinkyTable
        retlw .15
        retlw .21
        retlw .15
        retlw .14
        retlw .11
        retlw .19
        retlw .23
        retlw .28
        retlw .29
        retlw .26
        retlw .12
        retlw .24
        retlw .15
        retlw .23
        retlw .16
        retlw .26
        retlw .36
        retlw .9
        retlw .13
        retlw .28
        retlw .30
        retlw .13
        retlw .36
        retlw .20
        retlw .23
        retlw .15
        retlw .7
        retlw .28
        retlw .15
        retlw .28
        retlw .16
        retlw .7
        retlw .13
        retlw .33
        retlw .16
        retlw .16
        retlw .23
        retlw .10
        retlw .10
        retlw .32
        retlw .24
        retlw .19
        retlw .18
        retlw .28
        retlw .24
        retlw .22
        retlw .24
        retlw .12
        retlw .7
        retlw .14
        retlw .20
        retlw .15
        retlw .35
        retlw .25
        retlw .30
        retlw .28
        retlw .18
        retlw .26
        retlw .18
        retlw .22
        retlw .13
        retlw .36
        retlw .11
        retlw .28
        retlw .7
        retlw .25
        retlw .24
        retlw .14
        retlw .16
        retlw .16
        retlw .13
        retlw .9
        retlw .14
        retlw .9
        retlw .26
        retlw .24
        retlw .30
        retlw .17
        retlw .11
        retlw .15
        retlw .18
        retlw .30
        retlw .17
        retlw .28
        retlw .13
        retlw .13
        retlw .35
        retlw .25
        retlw .19
        retlw .12
        retlw .13
        retlw .14
        retlw .24
        retlw .29
        retlw .25
        retlw .29
        retlw .14
        retlw .10
        retlw .15
        retlw .25
        retlw .7
        retlw .14
        retlw .30
        retlw .17
        retlw .20
        retlw .9
        retlw .14
        retlw .22
        retlw .10
        retlw .26
        retlw .30
        retlw .19
        retlw .32
        retlw .23
        retlw .14
        retlw .16
        retlw .28
        retlw .23
        retlw .19
        retlw .17
        retlw .22
        retlw .7
        retlw .18
        retlw .29
        retlw .7
        retlw .10
        retlw .7
        retlw .7
        retlw .10
        retlw .24
        retlw .6
        retlw .12
        retlw .23
        retlw .18
        retlw .9
        retlw .34
        retlw .16
        retlw .30
        retlw .23
        retlw .30
        retlw .12
        retlw .10
        retlw .16
        retlw .13
        retlw .23
        retlw .19
        retlw .15
        retlw .13
        retlw .22
        retlw .7
        retlw .11
        retlw .17
        retlw .23
        retlw .16
        retlw .33
        retlw .30
        retlw .10
        retlw .23
        retlw .13
        retlw .23
        retlw .20
        retlw .23
        retlw .28
        retlw .8
        retlw .19
        retlw .16
        retlw .30
        retlw .9
        retlw .29
        retlw .14
        retlw .9
        retlw .24
        retlw .9
        retlw .13
        retlw .10
        retlw .30
        retlw .20
        retlw .28
        retlw .35
        retlw .15
        retlw .16
        retlw .18
        retlw .20
        retlw .14
        retlw .28
        retlw .29
        retlw .28
        retlw .10
        retlw .24
        retlw .26
        retlw .25
        retlw .7
        retlw .13
        retlw .15
        retlw .26
        retlw .16
        retlw .21
        retlw .16
        retlw .9
        retlw .24
        retlw .33
        retlw .20
        retlw .23
        retlw .13
        retlw .19
        retlw .16
        retlw .8
        retlw .25
        retlw .20
        retlw .27
        retlw .14
        retlw .22
        retlw .16
        retlw .16
        retlw .21
        retlw .29
        retlw .5
        retlw .30
        retlw .26
        retlw .23
        retlw .18
        retlw .15
        retlw .23
        retlw .19
        retlw .13
        retlw .10
        retlw .24
        retlw .7
        retlw .29
        retlw .23
        retlw .33
        retlw .17
        retlw .12
        retlw .7
        retlw .24
        retlw .11
        retlw .7
        retlw .25
        retlw .10
        retlw .23
        retlw .21
        retlw .23
        retlw .9
        retlw .13
        retlw .18
        retlw .10
        retlw .14
        retlw .12
        retlw .36
        retlw .16
        retlw .13
        retlw .7
        retlw .22
        retlw .10
        retlw .21
        retlw .28
end
