		Title  "Enterprise Nacelle Program"

        list P=16F628
;
; ------------------
; CONFIGURATION FUSE
; ------------------
;
    INCLUDE "p16f628.inc"
	__CONFIG _BODEN_OFF & _CP_OFF & _DATA_CP_OFF & _PWRTE_OFF & _WDT_OFF & _LVP_OFF & _MCLRE_OFF & _INTRC_OSC_NOCLKOUT

;;; This program controls LEDs for a model of the Enterprise from
;;; Star Trek.  It operates one nacelle worth of LEDs (10 LEDs) on
;;; lines 0-4 of ports A and B, and also drives the navigational blinkers
;;; on RA7.
;;; This is an in-progress snapshot.  The dual inverted chaser has been
;;; implemented, along with 25% intensity for any LED next to a LED that
;;; the chaser has turned off.  Still pending: Random blinky on PortA,
;;; navigational blinkers

Temp equ 0x20
DelayL equ 0x21
DelayM equ 0x22
Count equ 0x23
LedSelect equ 0x24
LedMask equ 0x25
BufferA1 equ 0x26
BufferA2 equ 0x27
BufferB1 equ 0x28
BufferB2 equ 0x29


ClearFlags Macro
        movlw 0x0f8
        andwf STATUS, w
        movwf STATUS
        endm

;
; -------------
; PROGRAM START
; -------------
;
	org 0h			; startup address = 0000

        clrf PORTA
        clrf PORTB
        movlw 0x07
        movwf CMCON
        bcf STATUS, RP1
	bsf STATUS,RP0		; set RP0 for RAM page 1
	movlw b'00000000'		; all PortA = outputs
	movwf TRISA
	movlw b'00000000'		; all PortB = outputs
	movwf TRISB
	bcf STATUS,RP0		; set RP0 for RAM page 0
        movlw 0x0a
        movwf Count
        movlw 0xff
        movwf BufferA1
        movwf BufferA2
        movwf BufferB1
        movwf BufferB2

MainLoop
        call Delay10ms
        decf Count, 1
        movf Count, w
        movwf LedSelect
        rrf LedSelect, 1

        movlw 0
        movwf BufferA1
        movwf BufferB1

        movlw 1
        movwf LedMask
        ClearFlags
        movf LedSelect, w
LedMaskLoop
        btfsc STATUS, Z
        goto LedMaskDone
        rlf LedMask, f
        decf LedSelect, f
        goto LedMaskLoop
LedMaskDone
        btfsc Count, 0
        goto SetBufferB
SetBufferA
        movf LedMask, w
        movwf BufferA1
        movwf Temp
        bcf STATUS, C
        rlf LedMask, f
        rlf LedMask, f
        bcf STATUS, C
        rrf Temp, f
        rrf Temp, f
        rrf Temp, f
        movf Temp, w
        iorwf LedMask, w
        andlw 0x1f
        movwf BufferB1
        goto SetPorts
SetBufferB
        movf LedMask, w
        movwf BufferB1
        movwf Temp
        bcf STATUS, C
        rlf LedMask, f
        rlf LedMask, f
        rlf LedMask, f
        bcf STATUS, C
        rrf Temp, f
        rrf Temp, f
        movf Temp, w
        iorwf LedMask, w
        andlw 0x1f
        movwf BufferA1
        goto SetPorts
SetPorts
        ;; BufferA2/B2 has the same LEDs blanked out as A1/B1,
        ;; but the neighbors of these blanked-out LEDs are
        ;; blanked-out, too.
        movf BufferA1, w
        movwf BufferA2
        movwf Temp
        bcf STATUS, C
        rlf Temp, f
        rlf Temp, f
        movf Temp, w
        rlf Temp, f
        iorwf Temp, w
        iorwf BufferA2, f

        movf BufferA1, w
        movwf Temp
        bcf STATUS, C
        rrf Temp, f
        rrf Temp, f
        movf Temp, w
        rrf Temp, f
        iorwf Temp, w
        iorwf BufferA2, f
        movf BufferA2, w
        andlw 0x1f
        movwf BufferA2

        movf BufferB1, w
        movwf BufferB2
        movwf Temp
        bcf STATUS, C
        rlf Temp, f
        rlf Temp, f
        movf Temp, w
        rlf Temp, f
        iorwf Temp, w
        iorwf BufferB2, f

        movf BufferB1, w
        movwf Temp
        bcf STATUS, C
        rrf Temp, f
        rrf Temp, f
        movf Temp, w
        rrf Temp, f
        iorwf Temp, w
        iorwf BufferB2, f
        movf BufferB2, w
        andlw 0x1f
        movwf BufferB2

        movlw 0xff
        xorwf BufferA1, f
        xorwf BufferA2, f
        xorwf BufferB1, f
        xorwf BufferB2, f

        ClearFlags
        movf Count, w
        btfss STATUS, Z
        goto MainLoop
        movlw 0x0a
        movwf Count
        goto MainLoop


;;; Delay10ms: Busy-wait for approx. 10ms while alternately copying
;;; Buffer1 and Buffer2 to the output ports.  Buffer1 gets about 25% time
;;; and Buffer2 gets about 75% time.  (This basically gives two intensity
;;; levels for LEDs in addition to the "off" state: 25% and 100%.  50%
;;; was difficult to distinguish from 100%, so probably 75% is similarly
;;; useless.
Delay10ms
        clrf DelayL
        movlw 0x08
        movwf DelayM
Wait1
        movf DelayL, w
        andlw 0x03
        btfss STATUS, Z
        goto Flicker2
Flicker1
        movf BufferA1, w
        movwf PORTA
        movf BufferB1, w
        movwf PORTB
        goto FlickerDone
Flicker2
        movf BufferA2, w
        movwf PORTA
        movf BufferB2, w
        movwf PORTB
FlickerDone
        decfsz DelayL, 1
        goto Wait1
        decfsz DelayM, 1
        goto Wait1
        return


        end
